package aktbus.core.fipa;

import aktbus.core.RDFAktMessage;
import aktbus.core.AKTVOCAB;

import java.io.*;
import java.util.*;
import java.net.*;
import sunlabs.brazil.util.http.*;
import java.lang.reflect.Constructor;

// Jena packages
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

public class RDFFipaMessage extends RDFAktMessage implements FipaMessageInterface {
    
    static String HTTP_VERSION="1.1";	//HTTP version string
    static final int BUFFER_SIZE=10240;
    
    // CONSTRUCTORS
    //--------------
    
    public RDFFipaMessage() throws RDFException, Exception { 
	this("", "", "");
    }
   
    public RDFFipaMessage(URL sender,URL receiver,String content) throws RDFException, Exception {
	this(sender.toString(), receiver.toString(), content);
    }

    // Expects a full message as argument (eg. including akt header)
    public RDFFipaMessage(String message_to_parse) throws RDFException, Exception {
	super(message_to_parse);
    }

    public RDFFipaMessage(String sender,String receiver, String content) throws RDFException, Exception {

	// Create a normal RDFAktMessage
	super(sender, receiver, "");

	// Remove SimpleMessage from model (just for neatness)
	Resource contentResource = this.get_content();
	Statement oldStatement = contentResource.getProperty(AKTVOCAB.hasSimpleMessage);
	this.remove(oldStatement);

	// In case the content supplied is null, set performative to some default.
	//this.set_performative("inform");

	// Create content from simple_content argument
	this.set_simple_message(content);
   
    }	

    // METHODS TO MANIPULATE MESSAGE
    //------------------------------
      
    // Method is overridden to return the content as a string.
    // String is a proper FIPA message
    public String get_simple_message() {
	try {	   
	    String returnString = "(";
	    // No message will be returned if no performative is found.
	    if ((get_performative() != "") && (get_performative() != "SimpleContent"))
		returnString = returnString + get_performative() + " ";
	    else return "";
	    if (get_fipa_sender() != "")
		returnString = returnString + ":sender " + get_fipa_sender() + " ";
	    if (get_fipa_receiver() != "")
		returnString = returnString + ":receiver " + get_fipa_receiver() + " ";
	    if (get_fipa_content() != "")
	    returnString = returnString + ":content \"" + get_fipa_content() + "\" ";
	    if (get_reply_to() != "")
		returnString = returnString + ":reply-to " + get_reply_to() + " ";
	    if (get_in_reply_to() != "")
		returnString = returnString + ":in-reply-to " + get_in_reply_to() + " ";
	    if (get_language() != "")
		returnString = returnString + ":language " + get_language() + " ";
	    if (get_encoding() != "")
		returnString = returnString + ":encoding " + get_encoding() + " ";
	    if (get_ontology() != "")
		returnString = returnString + ":ontology " + get_ontology() + " ";
	    if (get_reply_with() != "")
		returnString = returnString + ":reply-with " + get_reply_with() + " ";
	    if (get_reply_by() != "")
		returnString = returnString + ":reply-by " + get_reply_by() + " ";
	    if (get_conversation_id() != "")
		returnString = returnString + ":conversation-id " + get_conversation_id() + " ";
	    if (get_protocol() != "")
		returnString = returnString + ":protocol " + get_protocol() + " ";
	    returnString = returnString + ")";
	    return returnString;
	}
	catch (Exception e) {
	    return "";
	}
    }

    public java.lang.String get_performative() {
	try {	   
	    Resource content = get_content();
	    Resource perf =  content.getProperty(RDF.type).getResource();
	    if (perf.getLocalName().equals("SimpleContent"))
		return "";
	    else
		return perf.getLocalName();	   
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}	
    } 

    public java.lang.String get_fipa_sender() {
	try {	   
	    Resource content = get_content();
	    return content.getProperty(FIPAVOCAB.fipa_sender).getLiteral().toString();	  
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}		

    }

    public java.lang.String get_fipa_receiver() {
	try {
	    Resource content = get_content();
	    return content.getProperty(FIPAVOCAB.fipa_receiver).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}		
    }

    public java.lang.String get_fipa_content() {
	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.content).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }

    public java.lang.String get_reply_to() {
	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.reply_to).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }
 
    public java.lang.String get_in_reply_to() {
    	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.in_reply_to).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }
    
    public java.lang.String get_language() {
	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.language).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }
    
    public java.lang.String get_encoding() {
	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.encoding).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }
    
    public java.lang.String get_ontology() {
    	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.ontology).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }
  
    public java.lang.String get_reply_with() {
	try {	   
	    Resource content = get_content();	
	    return content.getProperty(FIPAVOCAB.reply_with).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }

    public java.lang.String get_reply_by() {
	try {	   
	    Resource content = get_content();
	    return content.getProperty(FIPAVOCAB.reply_by).getLiteral().toString();
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }

    public java.lang.String get_conversation_id() {
	try {	   
	    Resource content = get_content();
	    Resource id = content.getProperty(FIPAVOCAB.conversation_id).getResource();	  
	    return id.getURI();	   
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}
    }

    public java.lang.String get_protocol() {
	try {	   
	    Resource content = get_content();
	    Resource protocol = content.getProperty(FIPAVOCAB.protocol).getResource();	  
	    return protocol.getLocalName();	   
	}
	catch (RDFException e) {
	    //System.out.println(e.toString());
	    return "";
	}	
    }

    public void set_performative(java.lang.String performative) {
	try {	   
	    Resource content = get_content();
	    Statement oldStatement =  content.getProperty(RDF.type);
	    oldStatement.set(new ResourceImpl(FIPAVOCAB.getURI(), performative));	   
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(RDF.type, (new ResourceImpl(FIPAVOCAB.getURI(), performative)));
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_fipa_sender(java.lang.String sender) {
	try {
	    Resource content = get_content();
	    Statement oldStatement = content.getProperty(FIPAVOCAB.fipa_sender);
	    oldStatement.set(sender); 	    
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.fipa_sender, sender);
	    }
	    catch (Exception f) {
	System.out.println(f.toString()); 
	    }
	}
    }

    public void set_fipa_receiver(java.lang.String receiver) {
	try {
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.fipa_receiver);
	    oldStatement.set(receiver);	
	}
	catch (Exception e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.fipa_receiver, receiver);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_fipa_content(java.lang.String newContent) {
    	try {

	    // Remove xml header (if present)
	    newContent = newContent.trim();
	    if (newContent.startsWith("<?xml version="))
		newContent = newContent.substring(newContent.indexOf('>')+1);
	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.content);	
	    oldStatement.set(newContent);	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.content, this.createLiteral(newContent));
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_reply_to(java.lang.String reply_to) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.reply_to);
	    oldStatement.set(reply_to);	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.reply_to, reply_to);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_in_reply_to(java.lang.String in_reply_to) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.in_reply_to);
	    oldStatement.set(in_reply_to);	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.in_reply_to, in_reply_to);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_language(java.lang.String language) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.language);
	    oldStatement.set(language);	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.language, language);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_encoding(java.lang.String encoding) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.encoding);
	    oldStatement.set(encoding);	  
	}
	catch (RDFException e) {
	  try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.encoding, encoding);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}   
    }
    public void set_ontology(java.lang.String ontology) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.ontology);
	    oldStatement.set(ontology);	  
	}
	catch (RDFException e) {
	   try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.ontology, ontology);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }
    public void set_reply_with(java.lang.String reply_with) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.reply_with);
	    oldStatement.set(reply_with);	  
	}
	catch (RDFException e) {
	   try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.reply_with, reply_with);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }
    public void set_reply_by(java.lang.String reply_by) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.reply_by);
	    oldStatement.set(reply_by);	  
	}
	catch (RDFException e) {
	   try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.reply_by, reply_by);
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}    
    }

    public void set_conversation_id(java.lang.String conversation_id) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.conversation_id);
	    oldStatement.set(new ResourceImpl(conversation_id));	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.conversation_id, new ResourceImpl(conversation_id));
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}
    }

    public void set_protocol(java.lang.String protocol) {
	try {	  
	    Resource content = get_content();   
	    Statement oldStatement = content.getProperty(FIPAVOCAB.protocol);
	    oldStatement.set(new ResourceImpl(protocol));	  
	}
	catch (RDFException e) {
	    try {
		Resource content = get_content();
		content.addProperty(FIPAVOCAB.protocol, new ResourceImpl(protocol));
	    }
	    catch (Exception f) {
		System.out.println(f.toString()); 
	    }
	}	
    }

    // Method is overridden to set content from a string.
    // String should be a proper FIPA message
    // Very hacky - should really parse message in a tidier
    // manner.
    public void set_simple_message(java.lang.String message) {
	try {
	  
	    //Remove opening and closing brackets from message
	    if (message.startsWith("("))
		message = message.substring(1);
	    if (message.endsWith(")"))
		message = message.substring(0, message.length()-1);

	    StringTokenizer tokener = new StringTokenizer(message, " ");

	    
	    // Set performative	   
	    String tempToken = tokener.nextToken();
	    String value = "";
	    
	    //System.out.println(tempToken);
	    this.set_performative(tempToken);

	    // Set rest of message elements
	    while (tokener.hasMoreTokens()) 	
			{
		tempToken = tokener.nextToken(" "); 

		if (tempToken.startsWith(":sender")) {
		    //System.out.println(tempToken);		    
		    tempToken = tokener.nextToken();
		    value = "";
		    while ( ( (!(tempToken.startsWith(":"))) 
			      || ((tempToken.startsWith(":name")) || (tempToken.startsWith(":addresses"))) )
			    && (tokener.hasMoreTokens()) )			
		    {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_fipa_sender(value);
		}

		if (tempToken.startsWith(":receiver")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ( ( (!(tempToken.startsWith(":"))) 
			      || ((tempToken.startsWith(":name")) || (tempToken.startsWith(":addresses"))) )
			    && (tokener.hasMoreTokens()) )			
			{
			    value = value + " " + tempToken;
			    tempToken = tokener.nextToken();
			}
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_fipa_receiver(value);
		}

		if (tempToken.startsWith(":content")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    value = value.trim();
		    if ((value.startsWith("\"")) && (value.endsWith("\"")))
			value = value.substring(1, value.length()-1);
		    this.set_fipa_content(value);
		}

		if (tempToken.startsWith(":reply-to")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_reply_to(value);
		}

		if (tempToken.startsWith(":in-reply-to")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_in_reply_to(value);
		}

		if (tempToken.startsWith(":language")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }

		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_language(value);
		}

		if (tempToken.startsWith(":encoding")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_encoding(value);
		}

		if (tempToken.startsWith(":ontology")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value); 
		    this.set_ontology(value);
		}

		if (tempToken.startsWith(":reply-with")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_reply_with(value);
		}

		if (tempToken.startsWith(":reply-by")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_reply_by(value);
		}

		if (tempToken.startsWith(":conversation-id")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_conversation_id(value);
		}

		if (tempToken.startsWith(":protocol")) {
		    //System.out.println(tempToken);
		    tempToken = tokener.nextToken();
		    value = "";
		    while ((!(tempToken.startsWith(":"))) && (tokener.hasMoreTokens())) {
			value = value + " " + tempToken;
			tempToken = tokener.nextToken();
		    }
		    if ((!(tempToken.startsWith(":"))) && (!(tokener.hasMoreTokens())))
			value = value + " " + tempToken;
		    //System.out.println(value);
		    this.set_protocol(value);
		}
		
		} 
	}
	catch (Exception e) {
	System.out.println(e.toString()); 
	}
	}

    public void ugly_write(java.io.Writer out) {
	FipaPrettyWriter pretty = new FipaPrettyWriter();
	pretty.writeMessage(this, out);
    }  

    public void pretty_write(java.io.Writer out) {
	FipaPrettyWriter pretty = new FipaPrettyWriter();
	pretty.prettyWriteMessage(this, out);
    }   
    
}
